package com.supervision.util;

import com.alibaba.fastjson.JSON;
import com.supervision.model.dify.DifyChatReqVO;
import com.supervision.model.dify.DifyChatResVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.hc.client5.http.ClientProtocolException;
import org.apache.hc.client5.http.classic.methods.HttpPost;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.core5.http.HttpEntity;
import org.apache.hc.core5.http.HttpHeaders;
import org.apache.hc.core5.http.HttpStatus;
import org.apache.hc.core5.http.io.entity.EntityUtils;
import org.apache.hc.core5.http.io.entity.StringEntity;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;

import static com.supervision.common.constant.DifyConstants.METHOD_CHAT_MESSAGES;

@Component
@Slf4j
public class DifyApiUtil {

    @Value("${dify.url}")
    private String difyUrl;
    @Value("${dify.dataset-auth}")
    private String difyDatasetAuth;
    @Value("${dify.app-auth}")
    private String difyAppAuth;

    public DifyChatResVO chat(DifyChatReqVO difyChatReqVO) {
        DifyChatResVO difyChatResVO = null;
        try (CloseableHttpClient httpClient = HttpClients.createDefault()) {
            HttpPost httpPost = new HttpPost(difyUrl + METHOD_CHAT_MESSAGES);
            httpPost.setHeader(HttpHeaders.AUTHORIZATION, difyAppAuth);
            httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/json");
            log.info("发起对话：{}", difyChatReqVO);
            StringEntity entity = new StringEntity(com.alibaba.fastjson.JSONObject.toJSON(difyChatReqVO).toString(), StandardCharsets.UTF_8);
            httpPost.setEntity(entity);
            difyChatResVO = httpClient.execute(httpPost, response -> {
                final int status = response.getCode();
                if (status >= HttpStatus.SC_SUCCESS && status < HttpStatus.SC_REDIRECTION) {
                    final HttpEntity responseEntity = response.getEntity();
                    String responseStr = EntityUtils.toString(responseEntity);
                    log.info("发起对话成功!");
                    return JSON.parseObject(responseStr, DifyChatResVO.class);
                } else {
                    log.info("responseEntity {}", EntityUtils.toString(response.getEntity()));
                    throw new ClientProtocolException("Unexpected response status: " + status);
                }
            });
        } catch (Exception e) {
            log.error("发起对话失败", e);
        }
        return difyChatResVO;
    }

}
